@@ -28,14 +28,15 @@
 NTOOLTIP g_tip;
 LRESULT CALLBACK TipHookProc(int nCode, WPARAM wParam, LPARAM lParam);
 
-char g_mru_list[MRU_LIST_SIZE][MAX_PATH] = { NULL, NULL, NULL, NULL, NULL };
+char g_mru_list[MRU_LIST_SIZE][MAX_PATH] = { "", "", "", "", "" };
 
 extern NSCRIPTDATA g_sdata;
 extern char *compressor_names[];
 
-int SetArgv(char *cmdLine, int *argc, char ***argv)
+int SetArgv(const char *cmdLine, int *argc, char ***argv)
 {
-  char *p, *arg, *argSpace;
+  const char *p;
+  char *arg, *argSpace;
   int size, argSpaceSize, inquote, copy, slashes;
 
   size = 2;
@@ -168,9 +169,9 @@ void Items(HWND hwnd, int on){
       g_sdata.focused_hwnd = GetFocus();
   // Altered by Darren Owen (DrO) on 6/10/2003
   else
-    EnableWindow(GetDlgItem(hwnd,IDC_CLOSE),1);
+    EnableWindow(GetDlgItem(hwnd,IDCANCEL),1);
 
-  EnableWindow(GetDlgItem(hwnd,IDC_CLOSE),on);
+  EnableWindow(GetDlgItem(hwnd,IDCANCEL),on);
   // Altered by Darren Owen (DrO) on 6/10/2003
   if((!g_sdata.retcode && on) || !on)
     EnableWindow(GetDlgItem(hwnd,IDC_TEST),on);
@@ -267,25 +268,43 @@ void CompileNSISScript() {
     DragAcceptFiles(g_sdata.hwnd,TRUE);
     return;
   }
-  if (!g_sdata.appended) {
+  if (!g_sdata.compile_command) {
     if (s) GlobalFree(s);
     char *symbols = BuildSymbols();
     
     char compressor[40];
     if(lstrlen(g_sdata.compressor_name)) {
-      wsprintf(compressor,"/X\"SetCompressor /FINAL %s\" ",g_sdata.compressor_name);
+      wsprintf(compressor,"/X\"SetCompressor /FINAL %s\"",g_sdata.compressor_name);
     }
     else {
       lstrcpy(compressor,"");
     }
 
-    s = (char *)GlobalAlloc(GPTR, lstrlen(g_sdata.script)+lstrlen(symbols)+lstrlen(compressor)+sizeof(EXENAME)+sizeof(" /NOTIFYHWND  ")+20);
-    wsprintf(s,"%s %s%s /NOTIFYHWND %d %s",EXENAME,compressor,symbols,g_sdata.hwnd,g_sdata.script);
+    char *args = (char *) GlobalLock(g_sdata.script_cmd_args);
+
+    g_sdata.compile_command = (char *) GlobalAlloc(
+      GPTR,
+      /* makensis.exe        */ sizeof(EXENAME)                   + /* space */ 1 +
+      /* script path         */ lstrlen(g_sdata.script)           + /* space */ 1 +
+      /* script cmd args     */ lstrlen(args)  + /* space */ 1 +
+      /* defines /Dblah=...  */ lstrlen(symbols)                  + /* space */ 1 +
+      /* /XSetCompressor...  */ lstrlen(compressor)               + /* space */ 1 +
+      /* /NOTTIFYHWND + HWND */ sizeof("/NOTIFYHWND -4294967295") + /* space */ 1
+    );
+
+    wsprintf(
+      g_sdata.compile_command,
+      "%s %s %s /NOTIFYHWND %d %s -- \"%s\"",
+      EXENAME,
+      compressor,
+      symbols,
+      g_sdata.hwnd,
+      args,
+      g_sdata.script
+    );
+
+    GlobalUnlock(args);
     GlobalFree(symbols);
-    if (g_sdata.script_alloced) GlobalFree(g_sdata.script);
-    g_sdata.script_alloced = true;
-    g_sdata.script = s;
-    g_sdata.appended = TRUE;
   }
   GlobalFree(g_sdata.input_script);
   GlobalFree(g_sdata.output_exe);
@@ -299,10 +318,29 @@ void CompileNSISScript() {
   g_sdata.thread=CreateThread(NULL,0,MakeNSISProc,0,0,&id);
 }
 
+static bool InternalOpenRegSettingsKey(HKEY root, HKEY &key, bool create) {
+  if (create) {
+    if (RegCreateKey(root, REGKEY, &key) == ERROR_SUCCESS)
+      return true;
+  } else {
+    if (RegOpenKeyEx(root, REGKEY, 0, KEY_READ, &key) == ERROR_SUCCESS)
+      return true;
+  }
+  return false;
+}
+
+bool OpenRegSettingsKey(HKEY &hKey, bool create) {
+  if (InternalOpenRegSettingsKey(REGSEC, hKey, create))
+    return true;
+  if (InternalOpenRegSettingsKey(REGSECDEF, hKey, create))
+    return true;
+  return false;
+}
+
 void RestoreWindowPos(HWND hwnd) {
   HKEY hKey;
   WINDOWPLACEMENT p;
-  if (RegOpenKeyEx(REGSEC,REGKEY,0,KEY_READ,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey)) {
     DWORD l = sizeof(p);
     DWORD t;
     if ((RegQueryValueEx(hKey,REGLOC,NULL,&t,(unsigned char*)&p,&l)==ERROR_SUCCESS)&&(t == REG_BINARY)&&(l==sizeof(p))) {
@@ -352,7 +390,7 @@ void SaveWindowPos(HWND hwnd) {
   WINDOWPLACEMENT p;
   p.length = sizeof(p);
   GetWindowPlacement(hwnd, &p);
-  if (RegCreateKey(REGSEC,REGKEY,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey, true)) {
     RegSetValueEx(hKey,REGLOC,0,REG_BINARY,(unsigned char*)&p,sizeof(p));
     RegCloseKey(hKey);
   }
@@ -372,7 +410,7 @@ void DeleteSymbolSet(char *name)
 {
   if(name) {
     HKEY hKey;
-    if (RegOpenKeyEx(REGSEC,REGKEY,0,KEY_READ,&hKey) == ERROR_SUCCESS) {
+    if (OpenRegSettingsKey(hKey)) {
       char subkey[1024];
       wsprintf(subkey,"%s\\%s",REGSYMSUBKEY,name);
       RegDeleteKey(hKey,subkey);
@@ -386,7 +424,7 @@ char** LoadSymbolSet(char *name)
   HKEY hKey;
   HKEY hSubKey;
   char **symbols = NULL;
-  if (RegOpenKeyEx(REGSEC,REGKEY,0,KEY_READ,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey)) {
     char subkey[1024];
     if(name) {
       wsprintf(subkey,"%s\\%s",REGSYMSUBKEY,name);
@@ -400,7 +438,7 @@ char** LoadSymbolSet(char *name)
       DWORD t;
       DWORD bufSize;
       DWORD i = 0;
-      HGLOBAL hMem;
+      HGLOBAL hMem = NULL;
 
       while(TRUE) {
         l = 0;
@@ -448,7 +486,7 @@ void SaveSymbolSet(char *name, char **symbols)
   HKEY hKey;
   HKEY hSubKey;
   int n = 0;
-  if (RegCreateKey(REGSEC,REGKEY,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey, true)) {
     char subkey[1024];
     if(name) {
       wsprintf(subkey,"%s\\%s",REGSYMSUBKEY,name);
@@ -488,10 +526,13 @@ void SaveSymbolSet(char *name, char **symbols)
 }
 
 void ResetObjects() {
-  g_sdata.appended = FALSE;
+  if (g_sdata.compile_command)
+    GlobalFree(g_sdata.compile_command);
+
   g_sdata.warnings = FALSE;
   g_sdata.retcode = -1;
   g_sdata.thread = NULL;
+  g_sdata.compile_command = NULL;
 }
 
 void ResetSymbols() {
@@ -544,7 +585,6 @@ int InitBranding() {
     }
     char szBuf[1024];
     DWORD dwRead = 1;
-    DWORD dwExit = !STILL_ACTIVE;
     if (WaitForSingleObject(pi.hProcess,10000)!=WAIT_OBJECT_0) {
       return 0;
     }
@@ -573,7 +613,7 @@ void InitTooltips(HWND h) {
   g_tip.tip = CreateWindowEx(dwExStyle,TOOLTIPS_CLASS,NULL,dwStyle,0,0,0,0,h,NULL,GetModuleHandle(NULL),NULL);
   if (!g_tip.tip) return;
   g_tip.hook = SetWindowsHookEx(WH_GETMESSAGE,TipHookProc,NULL, GetCurrentThreadId());
-  AddTip(GetDlgItem(h,IDC_CLOSE),TEXT("Close MakeNSISW"));
+  AddTip(GetDlgItem(h,IDCANCEL),TEXT("Close MakeNSISW"));
   AddTip(GetDlgItem(h,IDC_TEST),TEXT("Test the installer generated by MakeNSISW"));
   AddToolBarTooltips();
 }
@@ -637,7 +677,7 @@ char* BuildSymbols()
   }
   else {
     buf = (char *)GlobalAlloc(GPTR, sizeof(char));
-    buf[0] = NULL;
+    buf[0] = '\0';
   }
 
   return buf;
@@ -682,7 +722,6 @@ BOOL IsValidFile(char *fname)
 void PushMRUFile(char* fname)
 {
   int i;
-  char buf[MAX_PATH+1];
   DWORD   rv;
   char*  file_part;
   char full_file_name[MAX_PATH+1];
@@ -691,16 +730,8 @@ void PushMRUFile(char* fname)
     return;
   }
 
-  if(fname[0] == '"') {
-    fname++;
-  }
-
-  lstrcpy(buf,fname);
-  if(buf[lstrlen(buf)-1] == '"') {
-    buf[lstrlen(buf)-1] = '\0';
-  }
   my_memset(full_file_name,0,sizeof(full_file_name));
-  rv = GetFullPathName(buf,sizeof(full_file_name),full_file_name,&file_part);
+  rv = GetFullPathName(fname,sizeof(full_file_name),full_file_name,&file_part);
   if (rv == 0) {
     return;
   }
@@ -720,8 +751,9 @@ void BuildMRUMenus()
   HMENU hMenu = g_sdata.fileSubmenu;
   int i;
   MENUITEMINFO mii;
-  char buf[MRU_DISPLAY_LENGTH+1];
+  char buf[MRU_DISPLAY_LENGTH + 5/*number*/ + 1/*null*/];
   char buf2[MRU_DISPLAY_LENGTH - 6];
+  char buf3[MRU_DISPLAY_LENGTH + 1];
   int n;
 
   for(i = 0; i < MRU_LIST_SIZE; i++) {
@@ -738,6 +770,7 @@ void BuildMRUMenus()
       mii.fMask = MIIM_ID | MIIM_TYPE | MIIM_STATE;
       mii.wID = IDM_MRU_FILE+i;
       mii.fType = MFT_STRING;
+      wsprintf(buf, "&%d ", i + 1);
       if(lstrlen(g_mru_list[i]) > MRU_DISPLAY_LENGTH) {
         char *p = my_strrchr(g_mru_list[i],'\\');
         if(p) {
@@ -747,23 +780,26 @@ void BuildMRUMenus()
             lstrcpyn(buf2,p,MRU_DISPLAY_LENGTH - 9);
             lstrcat(buf2,"...");
 
-            lstrcpyn(buf,g_mru_list[i],4);
+            lstrcpyn(buf3,g_mru_list[i],4);
+            lstrcat(buf,buf3);
             lstrcat(buf,"...\\");
             lstrcat(buf,buf2);
           }
           else {
-            lstrcpyn(buf,g_mru_list[i],(MRU_DISPLAY_LENGTH - lstrlen(p) - 3));
+            lstrcpyn(buf3,g_mru_list[i],(MRU_DISPLAY_LENGTH - lstrlen(p) - 3));
+            lstrcat(buf,buf3);
             lstrcat(buf,"...\\");
             lstrcat(buf,p);
           }
         }
         else {
-          lstrcpyn(buf,g_mru_list[i],(MRU_DISPLAY_LENGTH-2));
+          lstrcpyn(buf3,g_mru_list[i],(MRU_DISPLAY_LENGTH-2));
+          lstrcat(buf,buf3);
           lstrcat(buf,"...");
         }
       }
       else {
-        lstrcpy(buf, g_mru_list[i]);
+        lstrcat(buf, g_mru_list[i]);
       }
 
       mii.dwTypeData = buf;
@@ -795,8 +831,7 @@ void BuildMRUMenus()
 void LoadMRUFile(int position)
 {
   if (!g_sdata.thread && position >=0 && position < MRU_LIST_SIZE && g_mru_list[position][0]) {
-    g_sdata.script = (char *)GlobalAlloc(GPTR,lstrlen(g_mru_list[position])+3);
-    wsprintf(g_sdata.script,"\"%s\"",g_mru_list[position]);
+    SetScript(g_mru_list[position]);
     if(IsValidFile(g_mru_list[position])) {
       PushMRUFile(g_mru_list[position]);
     }
@@ -815,7 +850,7 @@ void RestoreMRUList()
   HKEY hSubKey;
   int n = 0;
   int i;
-  if (RegOpenKeyEx(REGSEC,REGKEY,0,KEY_READ,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey)) {
     if (RegCreateKey(hKey,REGMRUSUBKEY,&hSubKey) == ERROR_SUCCESS) {
       char buf[8];
       DWORD l;
@@ -843,7 +878,7 @@ void SaveMRUList()
   HKEY hKey;
   HKEY hSubKey;
   int i = 0;
-  if (RegCreateKey(REGSEC,REGKEY,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey, true)) {
     if (RegCreateKey(hKey,REGMRUSUBKEY,&hSubKey) == ERROR_SUCCESS) {
       char buf[8];
       for(i = 0; i < MRU_LIST_SIZE; i++) {
@@ -870,7 +905,7 @@ void RestoreCompressor()
 {
   HKEY hKey;
   NCOMPRESSOR v = COMPRESSOR_SCRIPT;
-  if (RegOpenKeyEx(REGSEC,REGKEY,0,KEY_READ,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey)) {
     char compressor_name[32];
     DWORD l = sizeof(compressor_name);
     DWORD t;
@@ -899,7 +934,7 @@ void SaveCompressor()
     n = (int)v;
   }
 
-  if (RegCreateKey(REGSEC,REGKEY,&hKey) == ERROR_SUCCESS) {
+  if (OpenRegSettingsKey(hKey, true)) {
     RegSetValueEx(hKey,REGCOMPRESSOR,0,REG_SZ,(unsigned char*)compressor_names[n],
                   lstrlen(compressor_names[n]));
     RegCloseKey(hKey);
