@@ -143,7 +143,9 @@ void ExecScript(int log) {
   g_to = 0;      // default is no timeout
   bOEM = FALSE;  // default is no OEM->ANSI conversion
 
-  g_hwndList = FindWindowEx(FindWindowEx(g_hwndParent,NULL,"#32770",NULL),NULL,"SysListView32",NULL);
+  g_hwndList = NULL;
+  if (g_hwndParent)
+    g_hwndList = FindWindowEx(FindWindowEx(g_hwndParent,NULL,"#32770",NULL),NULL,"SysListView32",NULL);
 
   // add space
   pExec = g_exec + lstrlen(g_exec);
@@ -185,7 +187,7 @@ params:
     DWORD dwExit = !STILL_ACTIVE;
     DWORD dwLastOutput;
     static char szBuf[1024];
-    HGLOBAL hUnusedBuf;
+    HGLOBAL hUnusedBuf = NULL;
     char *szUnusedBuf = 0;
 
     if (log) {
@@ -253,7 +255,7 @@ params:
           }
 
           if (!(log & 2)) {
-            while (p = my_strstr(p, "\t")) {
+            while ((p = my_strstr(p, "\t"))) {
               if ((int)(p - szUnusedBuf) > (int)(GlobalSize(hUnusedBuf) - TAB_REPLACE_SIZE - 1))
               {
                 *p++ = ' ';
