@@ -1,3 +1,19 @@
+/*
+ * Platform.h
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 1999-2007 Nullsoft and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ */
+
 #ifndef ___PLATFORM__H___
 #define ___PLATFORM__H___
 
@@ -28,13 +44,14 @@ typedef unsigned long ULONG;
 typedef long long INT64, LARGE_INTEGER;
 typedef unsigned long long UINT64, ULARGE_INTEGER;
 typedef int BOOL, *LPBOOL;
+typedef short VARIANT_BOOL;
 typedef void VOID;
 typedef void *LPVOID;
 typedef char CHAR, *PCHAR, *LPCH, *PCH, *NPSTR, *LPSTR, *PSTR;
 typedef unsigned char UCHAR;
 typedef const char *LPCCH, *PCSTR, *LPCSTR;
-typedef unsigned short WCHAR, *PWCHAR, *LPWCH, *PWCH, *NWPSTR, *LPWSTR, *PWSTR;
-typedef const unsigned short *LPCWCH, *PCWCH, *LPCWSTR, *PCWSTR;
+typedef unsigned short WCHAR, OLECHAR, *PWCHAR, *LPWCH, *PWCH, *NWPSTR, *LPWSTR, *PWSTR, *BSTR;
+typedef const unsigned short *LPCWCH, *PCWCH, *LPCWSTR, *PCWSTR, *LPCOLESTR;
 typedef unsigned int UINT_PTR;
 // basic stuff
 typedef void * HANDLE;
@@ -47,6 +64,24 @@ typedef unsigned long HBRUSH;
 #  define TRUE 1
 // more
 typedef WORD LANGID;
+// ULONGLONG
+#ifdef __GNUC__
+#define _HAVE_INT64
+#define _INTEGRAL_MAX_BITS 64
+#undef __int64
+#define __int64 long long
+#elif defined(__WATCOMC__) && (defined(_INTEGRAL_MAX_BITS) && _INTEGRAL_MAX_BITS >= 64 )
+#define _HAVE_INT64
+#endif /* __GNUC__/__WATCOMC */
+#if defined(_HAVE_INT64) || (defined(_INTEGRAL_MAX_BITS) && _INTEGRAL_MAX_BITS >= 64)
+typedef __int64 LONGLONG;
+typedef unsigned __int64 DWORDLONG;
+#else
+typedef double LONGLONG,DWORDLONG;
+#endif
+typedef LONGLONG *PLONGLONG;
+typedef DWORDLONG *PDWORDLONG;
+typedef DWORDLONG ULONGLONG,*PULONGLONG;
 #endif
 
 #ifndef __BIG_ENDIAN__
@@ -81,11 +116,13 @@ typedef WORD LANGID;
 #  define PLATFORM_PATH_SEPARATOR_C '\\'
 #  define OPT_STR "/"
 #  define OPT_C '/'
+#  define IS_OPT(a) (a[0]==OPT_C||a[0]=='-')
 #else
 #  define PLATFORM_PATH_SEPARATOR_STR "/"
 #  define PLATFORM_PATH_SEPARATOR_C '/'
 #  define OPT_STR "-"
 #  define OPT_C '-'
+#  define IS_OPT(a) (a[0]==OPT_C)
 #endif
 
 // attributes
@@ -125,14 +162,26 @@ typedef WORD LANGID;
 #  endif
 #endif
 
+#if defined(__GNUC__)
+#define UNUSED __attribute__((unused))
+#else
+#define UNUSED
+#endif
+
 // macros
 
 #ifndef _WIN32
 #  ifndef FIELD_OFFSET
 #    define FIELD_OFFSET(t,f) ((LONG)&(((t*)0)->f))
 #  endif
+#  ifndef MAKEINTRESOURCEA
+#    define MAKEINTRESOURCEA(i) ((LPSTR)((ULONG_PTR)((WORD)(i))))
+#  endif
+#  ifndef MAKEINTRESOURCEW
+#    define MAKEINTRESOURCEW(i) ((LPWSTR)((ULONG_PTR)((WORD)(i))))
+#  endif
 #  ifndef MAKEINTRESOURCE
-#    define MAKEINTRESOURCE(i) (LPSTR)((DWORD)((WORD)(i)))
+#    define MAKEINTRESOURCE MAKEINTRESOURCEA
 #  endif
 #  ifndef IMAGE_FIRST_SECTION
 #    define IMAGE_FIRST_SECTION(h) ( PIMAGE_SECTION_HEADER( (DWORD) h + \
@@ -419,6 +468,7 @@ typedef WORD LANGID;
 #  define MB_ICONQUESTION 32
 #  define MB_ICONEXCLAMATION 48
 #  define MB_ICONINFORMATION 64
+#  define MB_USERICON 128
 #  define MB_SETFOREGROUND 0x10000
 #  define MB_TOPMOST 0x40000
 #  define MB_RIGHT 0x80000
@@ -500,6 +550,10 @@ typedef WORD LANGID;
 #  define HKEY_DYN_DATA ((HKEY)0x80000006)
 #endif
 
+#ifndef KEY_WOW64_64KEY
+#  define KEY_WOW64_64KEY 0x100
+#endif
+
 // show modes
 
 #ifndef SW_SHOWNORMAL
@@ -715,7 +769,53 @@ typedef struct _IMAGE_OPTIONAL_HEADER {
   DWORD LoaderFlags;
   DWORD NumberOfRvaAndSizes;
   IMAGE_DATA_DIRECTORY DataDirectory[IMAGE_NUMBEROF_DIRECTORY_ENTRIES];
-} IMAGE_OPTIONAL_HEADER,*PIMAGE_OPTIONAL_HEADER;
+} IMAGE_OPTIONAL_HEADER32,*PIMAGE_OPTIONAL_HEADER32;
+typedef struct _IMAGE_OPTIONAL_HEADER64 {
+  WORD Magic;
+  BYTE MajorLinkerVersion;
+  BYTE MinorLinkerVersion;
+  DWORD SizeOfCode;
+  DWORD SizeOfInitializedData;
+  DWORD SizeOfUninitializedData;
+  DWORD AddressOfEntryPoint;
+  DWORD BaseOfCode;
+  ULONGLONG ImageBase;
+  DWORD SectionAlignment;
+  DWORD FileAlignment;
+  WORD MajorOperatingSystemVersion;
+  WORD MinorOperatingSystemVersion;
+  WORD MajorImageVersion;
+  WORD MinorImageVersion;
+  WORD MajorSubsystemVersion;
+  WORD MinorSubsystemVersion;
+  DWORD Win32VersionValue;
+  DWORD SizeOfImage;
+  DWORD SizeOfHeaders;
+  DWORD CheckSum;
+  WORD Subsystem;
+  WORD DllCharacteristics;
+  ULONGLONG SizeOfStackReserve;
+  ULONGLONG SizeOfStackCommit;
+  ULONGLONG SizeOfHeapReserve;
+  ULONGLONG SizeOfHeapCommit;
+  DWORD LoaderFlags;
+  DWORD NumberOfRvaAndSizes;
+  IMAGE_DATA_DIRECTORY DataDirectory[IMAGE_NUMBEROF_DIRECTORY_ENTRIES];
+} IMAGE_OPTIONAL_HEADER64,*PIMAGE_OPTIONAL_HEADER64;
+#ifdef _WIN64
+typedef IMAGE_OPTIONAL_HEADER64         IMAGE_OPTIONAL_HEADER;
+typedef PIMAGE_OPTIONAL_HEADER64        PIMAGE_OPTIONAL_HEADER;
+#else
+typedef IMAGE_OPTIONAL_HEADER32         IMAGE_OPTIONAL_HEADER;
+typedef PIMAGE_OPTIONAL_HEADER32        PIMAGE_OPTIONAL_HEADER;
+#endif
+#ifndef __BIG_ENDIAN__
+#  define IMAGE_NT_OPTIONAL_HDR32_MAGIC 0x10b
+#  define IMAGE_NT_OPTIONAL_HDR64_MAGIC 0x20b
+#else
+#  define IMAGE_NT_OPTIONAL_HDR32_MAGIC 0x0b01
+#  define IMAGE_NT_OPTIONAL_HDR64_MAGIC 0x0b02
+#endif
 typedef struct _IMAGE_NT_HEADERS {
   DWORD Signature;
   IMAGE_FILE_HEADER FileHeader;
@@ -749,6 +849,21 @@ typedef struct _IMAGE_EXPORT_DIRECTORY {
   DWORD AddressOfNames;
   DWORD AddressOfNameOrdinals;
 } IMAGE_EXPORT_DIRECTORY,*PIMAGE_EXPORT_DIRECTORY;
+typedef struct tagVS_FIXEDFILEINFO {
+  DWORD dwSignature;
+  DWORD dwStrucVersion;
+  DWORD dwFileVersionMS;
+  DWORD dwFileVersionLS;
+  DWORD dwProductVersionMS;
+  DWORD dwProductVersionLS;
+  DWORD dwFileFlagsMask;
+  DWORD dwFileFlags;
+  DWORD dwFileOS;
+  DWORD dwFileType;
+  DWORD dwFileSubtype;
+  DWORD dwFileDateMS;
+  DWORD dwFileDateLS;
+} VS_FIXEDFILEINFO;
 #  pragma pack()
 #endif
 
