@@ -41,16 +41,15 @@ public:
 
       int offset2 = rand() % BUF_SIZE;
       int size2 = rand() % (BUF_SIZE - offset2);
-      int size2forrelease = size2;
-      char *p2 = (char *) mmap.getmore(offset2, &size2forrelease);
+      char *p2 = (char *) mmap.getmore(offset2, size2);
 
       int minsize = min(size1, size2);
-      for (size_t j = 0; j < minsize; j++) {
+      for (int j = 0; j < minsize; j++) {
         CPPUNIT_ASSERT_EQUAL( p1[j], p2[j] );
       }
 
       mmap.release();
-      mmap.release(p2, size2forrelease);
+      mmap.release(p2, size2);
     }
   }
 
