@@ -1,24 +1,18 @@
 /*
-*  Copyright (C) 1999-2006 Nullsoft, Inc.
-*  Portions Copyright (C) 2002 Jeff Doozan
-*
-*  This software is provided 'as-is', without any express or implied warranty.
-*  In no event will the authors be held liable for any damages arising from the
-*  use of this software.
-*
-*  Permission is granted to anyone to use this software for any purpose, including
-*  commercial applications, and to alter it and redistribute it freely, subject to
-*  the following restrictions:
-*
-*  1. The origin of this software must not be misrepresented; you must not claim that
-*  you wrote the original software. If you use this software in a product, an
-*  acknowledgment in the product documentation would be appreciated but is not required.
-*
-*  2. Altered source versions must be plainly marked as such, and must not be
-*  misrepresented as being the original software.
-*
-*  3. This notice may not be removed or altered from any source distribution.
-*/
+ * Ui.c
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 1999-2007 Nullsoft, Jeff Doozan and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ */
 
 #include <windowsx.h>
 #include <shlobj.h>
@@ -238,9 +232,8 @@ FORCE_INLINE int NSISCALL ui_doinstall(void)
   //   http://msdn.microsoft.com/library/default.asp?url=/library/en-us/intl/nls_0xrn.asp
 
   LANGID (WINAPI *GUDUIL)();
-  static const char guduil[] = "GetUserDefaultUILanguage";
 
-  GUDUIL = myGetProcAddress("KERNEL32.dll", (char *) guduil);
+  GUDUIL = myGetProcAddress(MGA_GetUserDefaultUILanguage);
   if (GUDUIL)
   {
     // Windows ME/2000+
@@ -248,23 +241,22 @@ FORCE_INLINE int NSISCALL ui_doinstall(void)
   }
   else
   {
+    static const char reg_9x_locale[]     = "Control Panel\\Desktop\\ResourceLocale";
+    static const char reg_nt_locale_key[] = ".DEFAULT\\Control Panel\\International";
+    const char       *reg_nt_locale_val   = &reg_9x_locale[30]; // = "Locale" with opt
+
     *(DWORD*)state_language = CHAR4_TO_DWORD('0', 'x', 0, 0);
 
     {
       // Windows 9x
-      static const char reg_9x_locale[] = "Control Panel\\Desktop\\ResourceLocale";
-
-      myRegGetStr(HKEY_CURRENT_USER, reg_9x_locale, NULL, g_tmp);
+      myRegGetStr(HKEY_CURRENT_USER, reg_9x_locale, NULL, g_tmp, 0);
     }
 
     if (!g_tmp[0])
     {
       // Windows NT
       // This key exists on 9x as well, so it's only read if ResourceLocale wasn't found
-      static const char reg_nt_locale_key[] = ".DEFAULT\\Control Panel\\International";
-      static const char reg_nt_locale_val[] = "Locale";
-
-      myRegGetStr(HKEY_USERS, reg_nt_locale_key, reg_nt_locale_val, g_tmp);
+      myRegGetStr(HKEY_USERS, reg_nt_locale_key, reg_nt_locale_val, g_tmp, 0);
     }
 
     mystrcat(state_language, g_tmp);
@@ -285,7 +277,8 @@ FORCE_INLINE int NSISCALL ui_doinstall(void)
         (HKEY)header->install_reg_rootkey,
         GetNSISStringNP(header->install_reg_key_ptr),
         GetNSISStringNP(header->install_reg_value_ptr),
-        ps_tmpbuf
+        ps_tmpbuf,
+        0
       );
       if (ps_tmpbuf[0])
       {
@@ -380,21 +373,20 @@ FORCE_INLINE int NSISCALL ui_doinstall(void)
 
 #ifdef NSIS_CONFIG_LICENSEPAGE
     { // load richedit DLL
-      static char str1[]="RichEd20.dll";
-      static char str2[]="RichEdit20A";
-      if (!LoadLibrary(str1))
+      static const char riched20[]="RichEd20";
+      static const char riched32[]="RichEd32";
+      static const char richedit20a[]="RichEdit20A";
+      static const char richedit[]="RichEdit";
+      if (!LoadLibrary(riched20))
       {
-        *(WORD*)(str1+6) = CHAR2_TO_WORD('3','2');
-        LoadLibrary(str1);
+        LoadLibrary(riched32);
       }
 
       // make richedit20a point to RICHEDIT
-      if (!GetClassInfo(NULL,str2,&wc))
+      if (!GetClassInfo(NULL,richedit20a,&wc))
       {
-        str2[8]=0;
-        GetClassInfo(NULL,str2,&wc);
-        wc.lpszClassName = str2;
-        str2[8]='2';
+        GetClassInfo(NULL,richedit,&wc);
+        wc.lpszClassName = richedit20a;
         RegisterClass(&wc);
       }
     }
@@ -550,6 +542,11 @@ nextPage:
       EnableNext(pflags & PF_NEXT_ENABLE);
       EnableWindow(m_hwndCancel, pflags & PF_CANCEL_ENABLE);
 
+      if (pflags & PF_CANCEL_ENABLE)
+        EnableMenuItem(GetSystemMenu(hwndDlg, FALSE), SC_CLOSE, MF_BYCOMMAND | MF_ENABLED);
+      else
+        EnableMenuItem(GetSystemMenu(hwndDlg, FALSE), SC_CLOSE, MF_BYCOMMAND | MF_GRAYED);
+
       SendMessage(hwndtmp, BM_SETSTYLE, BS_PUSHBUTTON, TRUE);
 
       if (g_exec_flags.abort)
@@ -605,6 +602,8 @@ nextPage:
           SetWindowPos(m_curwnd,0,r.left,r.top,0,0,SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOZORDER);
 #ifdef NSIS_SUPPORT_CODECALLBACKS
           ExecuteCodeSegment(this_page->showfunc,NULL);
+          if (g_quit_flag)
+            return FALSE;
 #endif //NSIS_SUPPORT_CODECALLBACKS
           ShowWindow(m_curwnd,SW_SHOWNA);
           NotifyCurWnd(WM_NOTIFY_START);
@@ -643,18 +642,10 @@ skipPage:
     SetWindowLong(hwndDlg, DWL_MSGRESULT, FALSE);
     return TRUE;
   }
-  if (uMsg == WM_CLOSE && m_page == g_blocks[NB_PAGES].num - 1)
-  {
-    if (!IsWindowEnabled(m_hwndCancel))
-    {
-      uMsg = WM_COMMAND;
-      wParam = IDOK;
-    }
-  }
   if (uMsg == WM_COMMAND)
   {
     int id = LOWORD(wParam);
-    HWND hCtl = GetDlgItem(hwndDlg, id);
+    HWND hCtl = GetDlgItem(hwndDlg, id); // lParam might be NULL
     if (hCtl)
     {
       SendMessage(hCtl, BM_SETSTATE, FALSE, 0);
@@ -927,9 +918,7 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
     {
       typedef HRESULT (WINAPI *SHAutoCompletePtr)(HWND, DWORD);
       SHAutoCompletePtr fSHAutoComplete;
-      static const char shlwapi[] = "shlwapi.dll";
-      static const char shac[] = "SHAutoComplete";
-      fSHAutoComplete = (SHAutoCompletePtr) myGetProcAddress((char *) shlwapi, (char *) shac);
+      fSHAutoComplete = (SHAutoCompletePtr) myGetProcAddress(MGA_SHAutoComplete);
       if (fSHAutoComplete)
       {
         fSHAutoComplete(hDir, SHACF_FILESYSTEM);
@@ -958,11 +947,12 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
       if (idlist)
       {
         // free idlist
-        FreePIDL(idlist);
+        CoTaskMemFree(idlist);
 
         addtrailingslash(dir);
 
-        if (g_header->install_directory_auto_append)
+        if (g_header->install_directory_auto_append &&
+          dir == state_install_directory) // only append to $INSTDIR (bug #1174184)
         {
           const char *post_str = ps_tmpbuf;
           GetNSISStringTT(g_header->install_directory_auto_append);
@@ -1002,7 +992,7 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
     // Test for and use the GetDiskFreeSpaceEx API
     {
       BOOL (WINAPI *GDFSE)(LPCSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER) =
-          myGetProcAddress("KERNEL32.dll", "GetDiskFreeSpaceExA");
+          myGetProcAddress(MGA_GetDiskFreeSpaceExA);
       if (GDFSE)
       {
         ULARGE_INTEGER available64;
@@ -1159,17 +1149,14 @@ int NSISCALL TreeGetSelectedSection(HWND tree, BOOL mouse)
 }
 
 static LONG oldTreeWndProc;
+static LPARAM last_selected_tree_item;
 static DWORD WINAPI newTreeWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
-  static LPARAM last_item=-1;
   if (uMsg == WM_CHAR && wParam == VK_SPACE) {
     NotifyCurWnd(WM_TREEVIEW_KEYHACK);
     return 0;
   }
 #if defined(NSIS_SUPPORT_CODECALLBACKS) && defined(NSIS_CONFIG_ENHANCEDUI_SUPPORT)
-  if (uMsg == WM_DESTROY) {
-    last_item=-1;
-  }
 #ifndef NSIS_CONFIG_COMPONENTPAGE_ALTERNATIVE
   if (uMsg == WM_MOUSEMOVE) {
     if (IsWindowVisible(hwnd)) {
@@ -1179,9 +1166,9 @@ static DWORD WINAPI newTreeWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM l
   }
 #endif
   if (uMsg == WM_NOTIFY_SELCHANGE) {
-    if (last_item != lParam)
+    if (last_selected_tree_item != lParam)
     {
-      last_item = lParam;
+      last_selected_tree_item = lParam;
 
       mystrcpy(g_tmp, g_usrvars[0]);
 
@@ -1218,6 +1205,7 @@ static BOOL CALLBACK SelProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
 
     hBMcheck1=LoadBitmap(g_hInstance, MAKEINTRESOURCE(IDB_BITMAP1));
 
+    last_selected_tree_item=-1;
     oldTreeWndProc=SetWindowLong(hwndTree1,GWL_WNDPROC,(long)newTreeWndProc);
 
     hImageList = ImageList_Create(16,16, ILC_COLOR32|ILC_MASK, 6, 0);
@@ -1588,6 +1576,7 @@ static BOOL CALLBACK InstProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
       ShowWindow(insthwndbutton,SW_HIDE);
       if (!(g_flags&CH_FLAGS_DETAILS_NEVERSHOW)) ShowWindow(linsthwnd,SW_SHOWNA);
       else insthwndbutton=NULL;
+      SetActiveCtl(insthwnd2);
     }
 
     {
@@ -1610,7 +1599,7 @@ static BOOL CALLBACK InstProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
   {
     ShowWindow(insthwndbutton,SW_HIDE);
     ShowWindow(linsthwnd,SW_SHOWNA);
-    SetNextDef();
+    SetActiveCtl(linsthwnd);
   }
   if (uMsg == WM_NOTIFY_INSTPROC_DONE)
   {
@@ -1654,7 +1643,7 @@ static BOOL CALLBACK InstProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
         TPM_NONOTIFY|TPM_RETURNCMD,
         pt.x,
         pt.y,
-        0,linsthwnd,0))
+        0,hwndDlg,0))
       {
         int i,total = 1; // 1 for the null char
         LVITEM item;
@@ -1680,8 +1669,7 @@ static BOOL CALLBACK InstProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
         i = 0;
         do {
           item.pszText = ptr;
-          SendMessage(linsthwnd,LVM_GETITEMTEXT,i,(LPARAM)&item);
-          ptr += mystrlen(ptr);
+          ptr += SendMessage(linsthwnd,LVM_GETITEMTEXT,i,(LPARAM)&item);
           *(WORD*)ptr = CHAR2_TO_WORD('\r','\n');
           ptr+=2;
         } while (++i < count);
