@@ -1,3 +1,19 @@
+/*
+ * util.c
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 1999-2007 Nullsoft and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ */
+
 #include "../Platform.h"
 #include <shellapi.h>
 #include "util.h"
@@ -7,6 +23,7 @@
 #include "fileform.h"
 #include "exec.h"
 #include "ui.h"
+#include "resource.h"
 
 #ifdef NSIS_CONFIG_LOG
 #if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
@@ -33,27 +50,12 @@ NSIS_STRING g_usrvars[1] __attribute__((section (NSIS_VARS_SECTION)));
 #  endif
 #endif
 
-void NSISCALL FreePIDL(LPITEMIDLIST idl)
+HANDLE NSISCALL myCreateProcess(char *cmd)
 {
-  IMalloc *m;
-  SHGetMalloc(&m);
-  if (m)
-  {
-    m->lpVtbl->Free(m, idl);
-    m->lpVtbl->Release(m);
-  }
-}
-
-HANDLE NSISCALL myCreateProcess(char *cmd, char *dir)
-{
-  DWORD d;
   PROCESS_INFORMATION ProcInfo;
   static STARTUPINFO StartUp;
   StartUp.cb=sizeof(StartUp);
-  d=GetFileAttributes(dir);
-  if (d == INVALID_FILE_ATTRIBUTES || !(d&FILE_ATTRIBUTE_DIRECTORY))
-    dir=0;
-  if (!CreateProcess(NULL, cmd, NULL, NULL, FALSE, 0, NULL, dir, &StartUp, &ProcInfo))
+  if (!CreateProcess(NULL, cmd, NULL, NULL, FALSE, 0, NULL, NULL, &StartUp, &ProcInfo))
     return NULL;
   CloseHandle(ProcInfo.hThread);
   return ProcInfo.hProcess;
@@ -79,6 +81,19 @@ int NSISCALL my_GetDialogItemText(UINT idx, char *val)
 
 int NSISCALL my_MessageBox(const char *text, UINT type) {
   int _type = type & 0x001FFFFF;
+  static MSGBOXPARAMS mbp = {
+    sizeof(MSGBOXPARAMS),
+    0,
+    0,
+    0,
+    0,
+    0,
+    MAKEINTRESOURCE(IDI_ICON2),
+    0,
+    0,
+    0
+  };
+  
 #ifdef NSIS_CONFIG_SILENT_SUPPORT
   // default for silent installers
   if (g_exec_flags.silent && type >> 21)
@@ -87,7 +102,14 @@ int NSISCALL my_MessageBox(const char *text, UINT type) {
   // no silent or no default, just show
   if (g_exec_flags.rtl)
     _type ^= MB_RIGHT | MB_RTLREADING;
-  return MessageBox(g_hwnd, text, g_caption, _type);
+
+  mbp.hwndOwner = g_hwnd;
+  mbp.hInstance = g_hInstance;
+  mbp.lpszText = text;
+  mbp.lpszCaption = g_caption;
+  mbp.dwStyle = _type;
+  
+  return MessageBoxIndirect(&mbp);
 }
 
 void NSISCALL myDelete(char *buf, int flags)
@@ -149,7 +171,7 @@ void NSISCALL myDelete(char *buf, int flags)
           else
           {
             log_printf2("Delete: DeleteFile(\"%s\")",buf);
-            SetFileAttributes(buf,fd.dwFileAttributes&(~FILE_ATTRIBUTE_READONLY));
+            remove_ro_attr(buf);
             if (!DeleteFile(buf))
             {
 #ifdef NSIS_SUPPORT_MOVEONREBOOT
@@ -192,7 +214,7 @@ void NSISCALL myDelete(char *buf, int flags)
     {
       addtrailingslash(buf);
       log_printf2("RMDir: RemoveDirectory(\"%s\")",buf);
-      SetFileAttributes(buf,FILE_ATTRIBUTE_NORMAL);
+      remove_ro_attr(buf);
       if (!RemoveDirectory(buf))
       {
 #ifdef NSIS_SUPPORT_MOVEONREBOOT
@@ -238,7 +260,7 @@ char * NSISCALL findchar(char *str, char c)
   return str;
 }
 
-void NSISCALL trimslashtoend(char *buf)
+char * NSISCALL trimslashtoend(char *buf)
 {
   char *p = buf + mystrlen(buf);
   do
@@ -249,6 +271,8 @@ void NSISCALL trimslashtoend(char *buf)
   } while (p > buf);
 
   *p = 0;
+
+  return p + 1;
 }
 
 int NSISCALL validpathspec(char *ubuf)
@@ -326,7 +350,7 @@ int NSISCALL is_valid_instpath(char *s)
   return 1;
 }
 
-char * NSISCALL mystrstri(char *a, char *b)
+char * NSISCALL mystrstri(char *a, const char *b)
 {
   int l = mystrlen(b);
   while (mystrlen(a) >= l)
@@ -354,6 +378,13 @@ void NSISCALL mini_memcpy(void *out, const void *in, int len)
   }
 }
 
+void NSISCALL remove_ro_attr(char *file)
+{
+  int attr = GetFileAttributes(file);
+  if (attr != INVALID_FILE_ATTRIBUTES)
+    SetFileAttributes(file,attr&(~FILE_ATTRIBUTE_READONLY));
+}
+
 HANDLE NSISCALL myOpenFile(const char *fn, DWORD da, DWORD cd)
 {
   int attr = GetFileAttributes(fn);
@@ -389,7 +420,7 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
   BOOL fOk = 0;
   typedef BOOL (WINAPI *mfea_t)(LPCSTR lpExistingFileName,LPCSTR lpNewFileName,DWORD dwFlags);
   mfea_t mfea;
-  mfea=(mfea_t) myGetProcAddress("KERNEL32.dll","MoveFileExA");
+  mfea=(mfea_t) myGetProcAddress(MGA_MoveFileExA);
   if (mfea)
   {
     fOk=mfea(pszExisting, pszNew, MOVEFILE_DELAY_UNTIL_REBOOT|MOVEFILE_REPLACE_EXISTING);
@@ -401,7 +432,7 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
     static char wininit[1024];
     static char tmpbuf[1024];
     int cchRenameLine;
-    char *szRenameSec = "[Rename]\r\n";
+    static const char szRenameSec[] = "[Rename]\r\n";
     HANDLE hfile;
     DWORD dwFileSize;
     DWORD dwBytes;
@@ -425,11 +456,8 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
       return;
     cchRenameLine = wsprintf(szRenameLine,"%s=%s\r\n",tmpbuf,wininit);
 
-    GetWindowsDirectory(wininit, 1024-16);
-    mystrcat(wininit, "\\wininit.ini");
-    hfile = CreateFile(wininit,
-        GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_ALWAYS,
-        FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
+    GetNSISString(wininit, g_header->str_wininit);
+    hfile = myOpenFile(wininit, GENERIC_READ | GENERIC_WRITE, OPEN_ALWAYS);
 
     if (hfile != INVALID_HANDLE_VALUE)
     {
@@ -485,11 +513,11 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
 }
 #endif
 
-void NSISCALL myRegGetStr(HKEY root, const char *sub, const char *name, char *out)
+void NSISCALL myRegGetStr(HKEY root, const char *sub, const char *name, char *out, int x64)
 {
   HKEY hKey;
   *out=0;
-  if (RegOpenKeyEx(root,sub,0,KEY_READ,&hKey) == ERROR_SUCCESS)
+  if (RegOpenKeyEx(root,sub,0,KEY_READ|(x64?KEY_WOW64_64KEY:0),&hKey) == ERROR_SUCCESS)
   {
     DWORD l = NSIS_MAX_STRLEN;
     DWORD t;
@@ -565,7 +593,8 @@ char * NSISCALL mystrcat(char *out, const char *concat)
 
 char ps_tmpbuf[NSIS_MAX_STRLEN*2];
 
-#define SYSREGKEY "Software\\Microsoft\\Windows\\CurrentVersion"
+const char SYSREGKEY[]   = "Software\\Microsoft\\Windows\\CurrentVersion";
+const char QUICKLAUNCH[] = "\\Microsoft\\Internet Explorer\\Quick Launch";
 
 // Based on Dave Laundon's simplified process_string
 char * NSISCALL GetNSISString(char *outbuf, int strtab)
@@ -593,45 +622,31 @@ char * NSISCALL GetNSISString(char *outbuf, int strtab)
 
       if (nVarIdx == NS_SHELL_CODE)
       {
-        // NOTE 1: the code CSIDL_PRINTERS, is used for QUICKLAUNCH
-        // NOTE 2: the code CSIDL_BITBUCKET is used for COMMONFILES
-        // NOTE 3: the code CSIDL_CONTROLS is used for PROGRAMFILES
         LPITEMIDLIST idl;
-        char *append = 0;
 
-        int x = 0;
+        int x = 2;
 
-        *out = 0;
-
-        if (fldrs[2] == CSIDL_PRINTERS) // QUICKLAUNCH
+        if (g_exec_flags.all_user_var)
         {
-          append = "\\Microsoft\\Internet Explorer\\Quick Launch";
-          x = 2;
+          x = 4;
         }
-        else if (fldrs[0] == CSIDL_PROGRAM_FILES_COMMON)
+
+        if (fldrs[0] & 0x80)
         {
-          myRegGetStr(HKEY_LOCAL_MACHINE, SYSREGKEY, "CommonFilesDir", out);
-        }
-        else if (fldrs[0] == CSIDL_PROGRAM_FILES)
-        {
-          myRegGetStr(HKEY_LOCAL_MACHINE, SYSREGKEY, "ProgramFilesDir", out);
+          myRegGetStr(HKEY_LOCAL_MACHINE, SYSREGKEY, GetNSISStringNP(fldrs[0] & 0x3F), out, fldrs[0] & 0x40);
           if (!*out)
-            mystrcpy(out, "C:\\Program Files");
+            GetNSISString(out, fldrs[2]);
+          x = 0;
         }
         else if (fldrs[0] == CSIDL_SYSTEM)
         {
           GetSystemDirectory(out, NSIS_MAX_STRLEN);
+          x = 0;
         }
         else if (fldrs[0] == CSIDL_WINDOWS)
         {
           GetWindowsDirectory(out, NSIS_MAX_STRLEN);
-        }
-
-        if (!*out)
-        {
-          x = 4;
-          if (!g_exec_flags.all_user_var)
-            x = 2;
+          x = 0;
         }
 
         while (x--)
@@ -639,32 +654,36 @@ char * NSISCALL GetNSISString(char *outbuf, int strtab)
           if (!SHGetSpecialFolderLocation(g_hwnd, fldrs[x], &idl))
           {
             BOOL res = SHGetPathFromIDList(idl, out);
-            FreePIDL(idl);
+            CoTaskMemFree(idl);
             if (res)
             {
               break;
             }
           }
-          else
-            *out=0;
+          *out=0;
         }
 
-        if (*out && append)
+        if (*out)
         {
-          mystrcat(out, append);
+          // all users' version is CSIDL_APPDATA only for $QUICKLAUNCH
+          // for normal $APPDATA, it'd be CSIDL_APPDATA_COMMON
+          if (fldrs[2] == CSIDL_APPDATA)
+          {
+            mystrcat(out, QUICKLAUNCH);
+          }
         }
 
         validate_filename(out);
       }
       else if (nVarIdx == NS_VAR_CODE)
       {
-        if (nData == 27) // HWNDPARENT
+        if (nData == 29) // $HWNDPARENT
           myitoa(out, (unsigned int) g_hwnd);
         else
           mystrcpy(out, g_usrvars[nData]);
         // validate the directory name
-        if ((unsigned int)(nData - 21) < 6) {
-          // validate paths for $INSTDIR, $OUTDIR, $EXEDIR, $LANGUAGE, $TEMP and $PLUGINSDIR
+        if ((unsigned int)(nData - 21) < 7) {
+          // validate paths for $INSTDIR, $OUTDIR, $EXEDIR, $LANGUAGE, $TEMP, $PLUGINSDIR and $EXEPATH
           // $LANGUAGE is just a number anyway...
           validate_filename(out);
         }
@@ -866,15 +885,33 @@ WIN32_FIND_DATA * NSISCALL file_exists(char *buf)
   return NULL;
 }
 
-void * NSISCALL myGetProcAddress(char *dll, char *func)
+struct MGA_FUNC
 {
+  const char *dll;
+  const char *func;
+};
+
+struct MGA_FUNC MGA_FUNCS[] = {
+  {"KERNEL32", "GetDiskFreeSpaceExA"},
+  {"KERNEL32", "MoveFileExA"},
+  {"ADVAPI32", "RegDeleteKeyExA"},
+  {"ADVAPI32", "OpenProcessToken"},
+  {"ADVAPI32", "LookupPrivilegeValueA"},
+  {"ADVAPI32", "AdjustTokenPrivileges"},
+  {"KERNEL32", "GetUserDefaultUILanguage"},
+  {"SHLWAPI",  "SHAutoComplete"}
+};
+
+void * NSISCALL myGetProcAddress(const enum myGetProcAddressFunctions func)
+{
+  const char *dll = MGA_FUNCS[func].dll;
   HMODULE hModule = GetModuleHandle(dll);
   if (!hModule)
     hModule = LoadLibrary(dll);
   if (!hModule)
     return NULL;
 
-  return GetProcAddress(hModule, func);
+  return GetProcAddress(hModule, MGA_FUNCS[func].func);
 }
 
 void NSISCALL MessageLoop(UINT uCheckedMsg)
