@@ -1,3 +1,19 @@
+/*
+ * lang.cpp
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 1999-2007 Nullsoft and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ */
+
 #include "Platform.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -20,11 +36,13 @@ typedef enum {
   BOTH_STATIC = 3
 } STATICID;
 
-struct {
+struct NLFString {
   char *szLangStringName;
   char *szDefault;
   STATICID eStaticID;
-} NLFStrings[NLF_STRINGS] = {
+};
+
+NLFString NLFStrings[NLF_STRINGS] = {
   {"^Branding", "Nullsoft Install System %s", BOTH_STATIC},
   {"^SetupCaption", "$(^Name) Setup", INSTALL_STATIC},
   {"^UninstallCaption", "$(^Name) Uninstall", UNINSTALL_STATIC},
@@ -118,7 +136,8 @@ struct {
   {"^Giga", "G", BOTH_STATIC},
   {"^Font", "MS Shell Dlg", NONE_STATIC},
   {"^FontSize", "8", NONE_STATIC},
-  {"^RTL", "0", NONE_STATIC}
+  {"^RTL", "0", NONE_STATIC},
+  {"^Language", "English", NONE_STATIC}
 };
 
 // ==============
@@ -312,13 +331,12 @@ void CEXEBuild::InitLangTables() {
   keep_ref = true;
 }
 
-LanguageTable* CEXEBuild::GetLangTable(LANGID &lang) {
+LanguageTable* CEXEBuild::GetLangTable(LANGID &lang, bool create/*=true*/) {
   int nlt = lang_tables.getlen() / sizeof(LanguageTable);
   LanguageTable *nla = (LanguageTable*)lang_tables.get();
 
   lang = lang ? lang : last_used_lang;
-  last_used_lang = lang;
-  LanguageTable *table = 0;
+  LanguageTable *table = NULL;
 
   for (int i = 0; i < nlt; i++) {
     if (lang == nla[i].lang_id) {
@@ -326,7 +344,7 @@ LanguageTable* CEXEBuild::GetLangTable(LANGID &lang) {
       break;
     }
   }
-  if (!table) {
+  if (!table && create) {
     LanguageTable newtable;
 
     newtable.lang_id = lang;
@@ -339,9 +357,32 @@ LanguageTable* CEXEBuild::GetLangTable(LANGID &lang) {
     table = (LanguageTable*)lang_tables.get() + nlt;
   }
 
+  if (table) // update last used language if a table was loaded
+    last_used_lang = lang;
+
   return table;
 }
 
+char *CEXEBuild::GetLangNameAndCP(LANGID lang, unsigned int *codepage/*=NULL*/) {
+  LanguageTable *table = GetLangTable(lang, false);
+
+  if (table && table->nlf.m_bLoaded) {
+    if (codepage)
+      *codepage = table->nlf.m_uCodePage;
+
+    return table->nlf.m_szName;
+  }
+  else {
+    if (codepage)
+      *codepage = 1252; // English US
+
+    if (lang == 1033)
+      return "English";
+    else
+      return "???";
+  }
+}
+
 int CEXEBuild::DefineLangString(char *name, int process/*=-1*/) {
   int index, uindex, pos, ret, sn;
   pos = build_langstrings.get(name, &sn, &index, &uindex);
@@ -441,6 +482,158 @@ int CEXEBuild::SetInnerString(int id, char *string) {
   return ret;
 }
 
+int CEXEBuild::GenerateLangTable(LanguageTable *lt, int num_lang_tables) {
+  // Add all installer language strings
+  int i, j, l, tabsset;
+  struct langstring* lang_strings = NULL;
+  TinyGrowBuf *string_ptrs = new TinyGrowBuf[num_lang_tables];
+
+  tabsset = 1;
+  while (tabsset)
+  {
+    tabsset = 0;
+    for (i = num_lang_tables; i--; )
+    {
+      // Fill in default values for all used language strings that we can
+      FillLanguageTable(&lt[i]);
+      // Make sure the string lists are large enough
+      string_ptrs[i].set_zeroing(1);
+      if (!uninstall_mode)
+        string_ptrs[i].resize(build_langstring_num * sizeof(int));
+      else
+        string_ptrs[i].resize(ubuild_langstring_num * sizeof(int));
+    }
+
+    // For all current language strings
+    if (!uninstall_mode)
+      lang_strings = build_langstrings.sort_index(&l);
+    else
+      lang_strings = build_langstrings.sort_uindex(&l);
+
+    for (j = 0; j < l; j++)
+    {
+      int lang_string_index;
+
+      if (!uninstall_mode)
+        lang_string_index = lang_strings[j].index;
+      else
+        lang_string_index = lang_strings[j].uindex;
+
+      // Is this language string used (in the installer)?
+      if (lang_string_index >= 0)
+      {
+        // For each language
+        for (i = num_lang_tables; i--; )
+        {
+          // Get the current string pointer
+          int *ptr = (int *)string_ptrs[i].get() + lang_string_index;
+          // Not already set?
+          if (!*ptr)
+          {
+            // Get the language string and its name
+            const char *str = lt[i].lang_strings->get(lang_strings[j].sn);
+            const char *lsn = build_langstrings.offset2name(lang_strings[j].name);
+            if (!str || !*str)
+            {
+              // No string is defined; give a warning (for user strings only)
+              if (lsn[0] != '^')
+              {
+                if (lt[i].nlf.m_bLoaded)
+                  warning("LangString \"%s\" is not set in language table of language %s", lsn, lt[i].nlf.m_szName);
+                else
+                  warning("LangString \"%s\" is not set in language table of language %d", lsn, lt[i].lang_id);
+              }
+            }
+            else
+            {
+              // Add the language string to the string data block
+              char fn[1024];
+              sprintf(fn, "LangString %s", lsn);
+              curfilename = fn;
+              linecnt = lt[i].lang_id;
+              *ptr = add_string(str, lang_strings[j].process, (WORD) lt[i].nlf.m_uCodePage);
+              curfilename = 0;
+              // Indicate that we should check again for any newly referenced language strings
+              tabsset++;
+            }
+          }
+        }
+      }
+    }
+  }
+
+  // Optimize langstrings and check for recursion
+  for (i = num_lang_tables; i--; )
+  {
+    TinyGrowBuf rec;
+    int *lst = (int *)string_ptrs[i].get();
+
+    int langstring_num;
+
+    if (!uninstall_mode)
+      langstring_num = build_langstring_num;
+    else
+      langstring_num = ubuild_langstring_num;
+
+    for (j = 0; j < langstring_num; j++)
+    {
+      // Does this string reference another language string directly?
+      while (lst[j] < 0)
+      {
+        // Search through list of language string references
+        for (l = 0; (unsigned int)l < rec.getlen() / sizeof(int); l++)
+        {
+          if (((int*)rec.get())[l] == lst[j])
+          {
+            // We have the index of a recursive language string; now find the name
+            const char *name = "(unnamed)";
+            for (l = 0; l < langstring_num; l++)
+            {
+              int index;
+
+              if (!uninstall_mode)
+                index = lang_strings[l].index;
+              else
+                index = lang_strings[l].uindex;
+
+              if (lang_strings[l].index == j)
+              {
+                name = build_langstrings.offset2name(lang_strings[l].name);
+              }
+            }
+            ERROR_MSG("Error: LangString %s is recursive!\n", name);
+            delete [] string_ptrs;
+            return PS_ERROR;
+          }
+        }
+        // Add this reference to the list
+        rec.add(&lst[j], sizeof(int));
+        // and dereference it
+        lst[j] = lst[-lst[j] - 1];
+      }
+      rec.resize(0);
+    }
+  }
+
+  // Add language tables into their datablock
+  for (i = num_lang_tables; i--; )
+  {
+    cur_langtables->add(&lt[i].lang_id, sizeof(LANGID));
+    cur_langtables->add(&lt[i].dlg_offset, sizeof(int));
+    int rtl = lt[i].nlf.m_bRTL ? 1 : 0;
+    cur_langtables->add(&rtl, sizeof(int));
+    cur_langtables->add(string_ptrs[i].get(), string_ptrs[i].getlen());
+    string_ptrs[i].resize(0);
+  }
+
+  cur_header->blocks[NB_LANGTABLES].num = num_lang_tables;
+  cur_header->langtable_size = cur_langtables->getlen() / num_lang_tables;
+
+  delete [] string_ptrs;
+
+  return PS_OK;
+}
+
 int CEXEBuild::GenerateLangTables() {
   int i;
   LanguageTable *lt = (LanguageTable*)lang_tables.get();
@@ -474,14 +667,14 @@ int CEXEBuild::GenerateLangTables() {
       init_res_editor();
 
 #define ADD_FONT(id) { \
-        BYTE* dlg = res_editor->GetResource(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG); \
+        BYTE* dlg = res_editor->GetResourceA(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG); \
         if (dlg) { \
           CDialogTemplate td(dlg); \
           res_editor->FreeResource(dlg); \
-          td.SetFont(build_font, build_font_size); \
+          td.SetFont(build_font, (WORD) build_font_size); \
           DWORD dwSize; \
           dlg = td.Save(dwSize); \
-          res_editor->UpdateResource(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG, dlg, dwSize); \
+          res_editor->UpdateResourceA(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG, dlg, dwSize); \
           delete [] dlg; \
         } \
       }
@@ -529,11 +722,11 @@ int CEXEBuild::GenerateLangTables() {
         init_res_editor();
 
 #define ADD_FONT(id) { \
-          BYTE* dlg = res_editor->GetResource(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG); \
+          BYTE* dlg = res_editor->GetResourceA(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG); \
           if (dlg) { \
             CDialogTemplate td(dlg,lt[i].nlf.m_uCodePage); \
             res_editor->FreeResource(dlg); \
-            if (font) td.SetFont(font, lt[i].nlf.m_iFontSize); \
+            if (font) td.SetFont(font, (WORD) lt[i].nlf.m_iFontSize); \
             if (lt[i].nlf.m_bRTL) { \
               td.ConvertToRTL(); \
               DialogItemTemplate* dir = td.GetItem(IDC_DIR); \
@@ -544,7 +737,7 @@ int CEXEBuild::GenerateLangTables() {
             } \
             DWORD dwSize; \
             dlg = td.Save(dwSize); \
-            res_editor->UpdateResource(RT_DIALOG, MAKEINTRESOURCE(id+cur_offset), NSIS_DEFAULT_LANG, dlg, dwSize); \
+            res_editor->UpdateResourceA(RT_DIALOG, MAKEINTRESOURCE(id+cur_offset), NSIS_DEFAULT_LANG, dlg, dwSize); \
             delete [] dlg; \
           } \
         }
@@ -577,237 +770,22 @@ int CEXEBuild::GenerateLangTables() {
     }
   }
 
-  // Add all installer language strings
-  int j, l, tabsset;
-  struct langstring* lang_strings;
-  TinyGrowBuf *string_ptrs = new TinyGrowBuf[num_lang_tables];
-
-  tabsset = 1;
-  while (tabsset)
-  {
-    tabsset = 0;
-    for (i = num_lang_tables; i--; )
-    {
-      // Fill in default values for all used language strings that we can
-      FillLanguageTable(&lt[i]);
-      // Make sure the string lists are large enough
-      string_ptrs[i].set_zeroing(1);
-      string_ptrs[i].resize(build_langstring_num * sizeof(int));
-    }
-
-    // For all current language strings
-    lang_strings = build_langstrings.sort_index(&l);
-    for (j = 0; j < l; j++)
-    {
-      // Is this language string used (in the installer)?
-      if (lang_strings[j].index >= 0)
-      {
-        // For each language
-        for (i = num_lang_tables; i--; )
-        {
-          // Get the current string pointer
-          int *ptr = (int *)string_ptrs[i].get() + lang_strings[j].index;
-          // Not already set?
-          if (!*ptr)
-          {
-            // Get the language string and its name
-            const char *str = lt[i].lang_strings->get(lang_strings[j].sn);
-            const char *lsn = build_langstrings.offset2name(lang_strings[j].name);
-            if (!str || !*str)
-            {
-              // No string is defined; give a warning (for user strings only)
-              if (lsn[0] != '^')
-              {
-                if (lt[i].nlf.m_bLoaded)
-                  warning("LangString \"%s\" is not set in language table of language %s", lsn, lt[i].nlf.m_szName);
-                else
-                  warning("LangString \"%s\" is not set in language table of language %d", lsn, lt[i].lang_id);
-              }
-            }
-            else
-            {
-              // Add the language string to the string data block
-              char fn[1024];
-              sprintf(fn, "LangString %s", lsn);
-              curfilename = fn;
-              linecnt = lt[i].lang_id;
-              *ptr = add_string(str, lang_strings[j].process, lt[i].nlf.m_uCodePage);
-              curfilename = 0;
-              // Indicate that we should check again for any newly referenced language strings
-              tabsset++;
-            }
-          }
-        }
-      }
-    }
-  }
-
-  // Optimize langstrings and check for recursion
-  for (i = num_lang_tables; i--; )
-  {
-    TinyGrowBuf rec;
-    int *lst = (int *)string_ptrs[i].get();
-    for (j = 0; j < build_langstring_num; j++)
-    {
-      // Does this string reference another language string directly?
-      while (lst[j] < 0)
-      {
-        // Search through list of language string references
-        for (l = 0; (unsigned int)l < rec.getlen() / sizeof(int); l++)
-        {
-          if (((int*)rec.get())[l] == lst[j])
-          {
-            // We have the index of a recursive language string; now find the name
-            const char *name = "(unnamed)";
-            for (l = 0; l < build_langstring_num; l++)
-              if (lang_strings[l].index == j)
-                name = build_langstrings.offset2name(lang_strings[l].name);
-            ERROR_MSG("Error: LangString %s is recursive!\n", name);
-            delete [] string_ptrs;
-            return PS_ERROR;
-          }
-        }
-        // Add this reference to the list
-        rec.add(&lst[j], sizeof(int));
-        // and dereference it
-        lst[j] = lst[-lst[j] - 1];
-      }
-      rec.resize(0);
-    }
-  }
-
-  // Add language tables into their datablock
-  for (i = num_lang_tables; i--; )
-  {
-    build_langtables.add(&lt[i].lang_id, sizeof(LANGID));
-    build_langtables.add(&lt[i].dlg_offset, sizeof(int));
-    int rtl = lt[i].nlf.m_bRTL ? 1 : 0;
-    build_langtables.add(&rtl, sizeof(int));
-    build_langtables.add(string_ptrs[i].get(), string_ptrs[i].getlen());
-    string_ptrs[i].resize(0);
-  }
-
-  build_header.blocks[NB_LANGTABLES].num = num_lang_tables;
-  build_header.langtable_size = build_langtables.getlen() / num_lang_tables;
-
-#ifdef NSIS_CONFIG_UNINSTALL_SUPPORT
-  // Now do it all again, this time for the uninstaller
-  set_uninstall_mode(1);
-
-  tabsset = 1;
-  while (tabsset)
-  {
-    tabsset = 0;
-    for (i = num_lang_tables; i--; )
-    {
-      // Fill in default values for all used language strings that we can
-      FillLanguageTable(&lt[i]);
-      // Make sure the string lists are large enough
-      string_ptrs[i].set_zeroing(1);
-      string_ptrs[i].resize(ubuild_langstring_num * sizeof(int));
-    }
-
-    // For all current language strings
-    lang_strings = build_langstrings.sort_uindex(&l);
-    for (j = 0; j < l; j++)
-    {
-      // Is this language string used (in the uninstaller)?
-      if (lang_strings[j].uindex >= 0)
-      {
-        // For each language
-        for (i = num_lang_tables; i--; )
-        {
-          // Get the current string pointer
-          int *ptr = (int *)string_ptrs[i].get() + lang_strings[j].uindex;
-          // Not already set?
-          if (!*ptr)
-          {
-            // Get the language string and its name
-            const char *str = lt[i].lang_strings->get(lang_strings[j].sn);
-            const char *lsn = build_langstrings.offset2name(lang_strings[j].name);
-            if (!str || !*str)
-            {
-              // No string is defined; give a warning (for user strings only)
-              if (lsn[0] != '^')
-              {
-                if (lt[i].nlf.m_bLoaded)
-                  warning("LangString \"%s\" is not set in language table of language %s", lsn, lt[i].nlf.m_szName);
-                else
-                  warning("LangString \"%s\" is not set in language table of language %d", lsn, lt[i].lang_id);
-              }
-            }
-            else
-            {
-              // Add the language string to the string data block
-              char fn[1024];
-              sprintf(fn, "LangString %s", lsn);
-              curfilename = fn;
-              linecnt = lt[i].lang_id;
-              *ptr = add_string(str, lang_strings[j].process, lt[i].nlf.m_uCodePage);
-              curfilename = 0;
-              // Indicate that we should check again for any newly referenced language strings
-              tabsset++;
-            }
-          }
-        }
-      }
-    }
-  }
-
-  // Optimize langstrings and check for recursion
-  for (i = num_lang_tables; i--; )
-  {
-    TinyGrowBuf rec;
-    int *lst = (int *)string_ptrs[i].get();
-    for (j = 0; j < ubuild_langstring_num; j++)
-    {
-      // Does this string reference another language string directly?
-      while (lst[j] < 0)
-      {
-        // Search through list of language string references
-        for (l = 0; (unsigned int)l < rec.getlen() / sizeof(int); l++)
-        {
-          if (((int*)rec.get())[l] == lst[j])
-          {
-            // We have the index of a recursive language string; now find the name
-            const char *name = "(unnamed)";
-            for (l = 0; l < ubuild_langstring_num; l++)
-              if (lang_strings[l].uindex == j)
-                name = build_langstrings.offset2name(lang_strings[l].name);
-            ERROR_MSG("Error: LangString %s is recursive!\n", name);
-            delete [] string_ptrs;
-            return PS_ERROR;
-          }
-        }
-        // Add this reference to the list
-        rec.add(&lst[j], sizeof(int));
-        // and dereference it
-        lst[j] = lst[-lst[j] - 1];
-      }
-      rec.resize(0);
-    }
-  }
-
-  // Add language tables into their datablock
-  for (i = num_lang_tables; i--; )
-  {
-    ubuild_langtables.add(&lt[i].lang_id, sizeof(LANGID));
-    ubuild_langtables.add(&lt[i].dlg_offset, sizeof(int));
-    int rtl = lt[i].nlf.m_bRTL ? 1 : 0;
-    ubuild_langtables.add(&rtl, sizeof(int));
-    ubuild_langtables.add(string_ptrs[i].get(), string_ptrs[i].getlen());
-    string_ptrs[i].resize(0);
-  }
-
-  build_uninst.blocks[NB_LANGTABLES].num = num_lang_tables;
-  build_uninst.langtable_size = ubuild_langtables.getlen() / num_lang_tables;
+  int orig_uninstall_mode = uninstall_mode;
 
   set_uninstall_mode(0);
+  if (GenerateLangTable(lt, num_lang_tables) != PS_OK)
+    return PS_ERROR;
+
+#ifdef NSIS_CONFIG_UNINSTALL_SUPPORT
+  set_uninstall_mode(1);
+  if (GenerateLangTable(lt, num_lang_tables) != PS_OK)
+    return PS_ERROR;
 #endif
 
+  set_uninstall_mode(orig_uninstall_mode);
+
   SCRIPT_MSG("Done!\n");
 
-  delete [] string_ptrs;
   return PS_OK;
 }
 
@@ -864,7 +842,7 @@ void CEXEBuild::FillLanguageTable(LanguageTable *table) {
           }
           else if (i == NLF_FONTSIZE)
           {
-            int font_size = *build_font ? build_font_size : table->nlf.m_iFontSize;
+            WORD font_size = *build_font ? (WORD) build_font_size : (WORD) table->nlf.m_iFontSize;
             if (font_size)
             {
               char temp[64];
@@ -883,7 +861,7 @@ void CEXEBuild::FillLanguageTable(LanguageTable *table) {
 }
 
 char SkipComments(FILE *f) {
-  char c;
+  int c;
   while ((c = fgetc(f))) {
     while (c == '\n' || c == '\r') {
       c = fgetc(f); // Skip empty lines
@@ -895,7 +873,7 @@ char SkipComments(FILE *f) {
     }
     else break;
   }
-  return c;
+  return (char) c;
 }
 
 #ifndef _WIN32
@@ -972,6 +950,9 @@ LanguageTable * CEXEBuild::LoadLangFile(char *filename) {
     warning_fl("%s language file version doesn't match. Using default English texts for missing strings.", nlf->m_szName);
   }
 
+  // set ^Language
+  nlf->m_szStrings[NLF_LANGUAGE] = strdup(nlf->m_szName);
+
   int temp;
 
   // Get font
