@@ -1,3 +1,19 @@
+/*
+ * lineparse.h
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 1999-2007 Nullsoft and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ */
+
 #ifndef _LINEPARSE_H_
 #define _LINEPARSE_H_
 
@@ -7,7 +23,8 @@ class LineParser {
     LineParser(bool bCommentBlock);
     virtual ~LineParser();
 
-    bool InCommentBlock();
+    bool inComment();
+    bool inCommentBlock();
     int parse(char *line, int ignore_escaping=0); // returns -1 on error
     int getnumtokens();
     void eattoken();
@@ -23,7 +40,8 @@ class LineParser {
 
     int m_eat;
     int m_nt;
-    bool m_bCommentBlock;
+    bool m_incommentblock;
+    bool m_incomment;
     char **m_tokens;
 };
 #endif//_LINEPARSE_H_
