@@ -1,3 +1,19 @@
+/*
+ * util.h
+ * 
+ * This file is a part of NSIS.
+ * 
+ * Copyright (C) 1999-2007 Nullsoft and Contributors
+ * 
+ * Licensed under the zlib/libpng license (the "License");
+ * you may not use this file except in compliance with the License.
+ * 
+ * Licence details can be found in the file COPYING.
+ * 
+ * This software is provided 'as-is', without any express or implied
+ * warranty.
+ */
+
 #ifndef _UTIL_H_
 #define _UTIL_H_
 
@@ -30,11 +46,10 @@ unsigned char* generate_uninstall_icon_data(const char* filename, size_t &size);
 int generate_unicons_offsets(unsigned char* exeHeader, size_t exeHeaderSize, unsigned char* uninstIconData);
 #endif//NSIS_CONFIG_UNINSTALL_SUPPORT
 
-// returns the number of WCHARs in str including null charcter
-size_t WCStrLen(const WCHAR* szwStr);
-
 size_t my_strftime(char *s, size_t max, const char  *fmt, const struct tm *tm);
 
+bool GetDLLVersion(const std::string& filepath, DWORD& high, DWORD& low);
+
 std::string get_full_path(const std::string& path);
 std::string get_dir_name(const std::string& path);
 std::string get_file_name(const std::string& path);
